package rrf.pingauges2;


import java.util.ArrayList;

/**
 * Created by Robert R. Fenichel on 12/16/2015.
 * This module holds all the math.
 */
public class Engine
{
  public static final int fitSnug      = 0;
  public static final int fitLoose     = 1;
  public static final int fitInfinite  = 2;
  public static final int fitOffside   = 3;
  public static final int fitUndefined = 4;

  public static int LastFit;
  public static double ClosestFit;
  public static double FoundNomBigD;
  public static double FoundNomMiddleD;
  public static double FoundNomSmallD;
  private static int FoundBigDIndex; // 1-based

  public static double CircumscribeThree(double D1, double D2, double D3)
    {
      double K1;
      double K2;
      double K3;
      double K4;
      double Result;

      Result = 0;
      if ((D1 <= 0) || (D2 <= 0) || (D3 <= 0))
        { LastFit = fitUndefined; }
      else
        { // nondegenerate
          LastFit = fitSnug;
          K1 = 2.0 / D1;
          K2 = 2.0 / D2;
          K3 = 2.0 / D3;
          K4 = K1 + K2 + K3 - 2.0 * Math.sqrt(K1 * K2 + K2 * K3 + K3 * K1);

          if (K4 == 0)
            { LastFit = fitInfinite; }
          else
            { Result = -2.0 / K4; }

          if (K4 > 0)
            { LastFit = fitOffside; }
          else if (Result > D1 + D2 + D3)
            { LastFit = fitLoose; }
        } // nondegenerate

      return Result;
    } // CircumscribeThree

  public static String FormatTuple(double NomOD1, double NomOD2, double NomOD3)
    {
      return String.format("(%s, %s, %s) ⇒ ",
                           dmPinGauges.FormatPin(NomOD1),
                           dmPinGauges.FormatPin(NomOD2),
                           dmPinGauges.FormatPin(NomOD3)
                          );
    } // FormatResult

  public static String FormatTupleHole(double NomOD1, double NomOD2, double NomOD3,
                                       double Hole)
    { return String.format("%s%.4f", FormatTuple(NomOD1, NomOD2, NomOD3), Hole); }

  private static double NthPinOD(int N)
    { // try to minimize accumulated rounding; N is 1-based
      return dmPinGauges.TheConfiguration.MinNomPinOD + (N - 1) * dmPinGauges.TheConfiguration.PinStep
              + dmPinGauges.TheConfiguration.PinOffset;
    } // NthPinOD

  public static ArrayList<String> ApproachFromBelow()
    { // return a new array list
      double ReducedBigD;
      double TentativeHole;
      double Middle = FoundNomMiddleD + dmPinGauges.TheConfiguration.PinOffset;
      double Small  = FoundNomSmallD  + dmPinGauges.TheConfiguration.PinOffset;
      ArrayList<String> TheArrayList = new ArrayList<String>();
      int Runner = FoundBigDIndex % 100;

      while (Runner < FoundBigDIndex)
        { ReducedBigD = NthPinOD(Runner);
          TentativeHole = CircumscribeThree(ReducedBigD, Middle, Small);
          if (LastFit == fitSnug)
            { TheArrayList.add(FormatTupleHole(ReducedBigD-dmPinGauges.TheConfiguration.PinOffset,
                                               FoundNomMiddleD,
                                               FoundNomSmallD,
                                               TentativeHole));
            }
          Runner = Runner + 100;
        }

      Runner = Runner - 90;
      while (Runner < FoundBigDIndex)
        { ReducedBigD = NthPinOD(Runner);
          TentativeHole = CircumscribeThree(ReducedBigD, Middle, Small);
          if (LastFit == fitSnug)
            { TheArrayList.add(FormatTupleHole(ReducedBigD-dmPinGauges.TheConfiguration.PinOffset,
                                               FoundNomMiddleD,
                                               FoundNomSmallD,
                                               TentativeHole));
            }
          Runner = Runner + 10;
        }

      Runner = Runner - 9;
      while (Runner <= FoundBigDIndex)
        { ReducedBigD = NthPinOD(Runner);
          TentativeHole = CircumscribeThree(ReducedBigD, Middle, Small);
          if (LastFit == fitSnug)
            { TheArrayList.add(FormatTupleHole(ReducedBigD-dmPinGauges.TheConfiguration.PinOffset,
                                               FoundNomMiddleD,
                                               FoundNomSmallD,
                                               TentativeHole));
            }
          Runner = Runner + 1;
        }

      return TheArrayList;
    } // ApproachFromBelow

  public static boolean FoundBestFit(double HoleOD)
    {
      double Delta = 9999.9;
      double HighAcceptable = HoleOD + dmPinGauges.TheConfiguration.Tightest;
      double LowAcceptable  = HoleOD + dmPinGauges.TheConfiguration.Loosest;
      double RangeMiddle    = (LowAcceptable + HighAcceptable) / 2.0;
      boolean FoundSolution = false;
      double BigD;
      double MiddleD;
      double OldCircumscriber;
      double SmallD;
      double Circumscriber;
      double NewDelta;

      double Span = dmPinGauges.TheConfiguration.MaxNomPinOD - dmPinGauges.TheConfiguration.MinNomPinOD;
      int MaxIndex = (int) (Span / dmPinGauges.TheConfiguration.PinStep +0.5) + 1;

      for (int BigIndex = MaxIndex; BigIndex > 0; BigIndex--)
        { // big pin chosen
          BigD = NthPinOD(BigIndex);

          if (BigD < HighAcceptable)
            { // BigD not too big
              for (int MiddleIndex = BigIndex - 1; MiddleIndex > 0; MiddleIndex--)
                { // middle pin chosen
                  MiddleD = NthPinOD(MiddleIndex);
                  OldCircumscriber = 9999.9;
                  if (BigD + MiddleD < HighAcceptable)
                    { // MiddleD not too big
                      for (int SmallIndex = MiddleIndex -1; SmallIndex > 0; SmallIndex--)
                        { // small pin chosen
                          SmallD = NthPinOD(SmallIndex);
                          if (BigD + MiddleD + SmallD < LowAcceptable)
                            { break; }
                          Circumscriber = CircumscribeThree(BigD, MiddleD, SmallD);
                          if ((Circumscriber > OldCircumscriber) || (LastFit != fitSnug))
                            { break; }
                          OldCircumscriber = Circumscriber;
                          NewDelta = Math.abs(RangeMiddle - Circumscriber);
                          if (NewDelta < Delta)
                            { // closing in
                              Delta = NewDelta;
                              if ((Circumscriber <= HighAcceptable) && (Circumscriber >= LowAcceptable))
                                { // got one
                                  ClosestFit = Circumscriber;
                                  FoundNomBigD    = BigD    - dmPinGauges.TheConfiguration.PinOffset;
                                  FoundNomMiddleD = MiddleD - dmPinGauges.TheConfiguration.PinOffset;
                                  FoundNomSmallD  = SmallD  - dmPinGauges.TheConfiguration.PinOffset;
                                  FoundSolution = true;
                                  FoundBigDIndex = BigIndex;
                                } // got one
                            } // closing in
                        } // small pin chosen
                    } // MiddleD not too big
                } // middle pin chosen
            } // BigD not too big
        } // big pin chosen

      return FoundSolution;
    } // FoundBestFit
} // Engine
