package rrf.pingauges2;

import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import java.util.Locale;

/**
 * Created by Robert R. Fenichel on 12/15/2015.
 */
public class HoleToPins extends AppCompatActivity
{
  private EditText edLoosest;
  private EditText edTightest;
  private static EditText edTargetHole;
  private static TextView lbBestFit;
  private static Button   btnFindPins;
  private static Button btnApproachFromBelow;

  private static final String PERIOD     = ".";
  private static final String MINUS      = "-";
  private static final String nullstring = "";

  @Override
  protected void onCreate(Bundle SavedState)
    {
      super.onCreate(SavedState);
      setContentView(R.layout.hole_to_pins);

      SetUpLoosestEditBox();
      SetUpTightestEditBox();
      SetUpTargetEditBox();
      SetUpActionButton();
      SetUpBestFit();
      SetUpApproachFromBelow();

      LoadUI();
    } // onCreate

  private void SetUpApproachFromBelow()
    {
      btnApproachFromBelow = (Button) findViewById(R.id.btnApproachFromBelow);
      btnApproachFromBelow.setEnabled(false);
      btnApproachFromBelow.setOnClickListener(new View.OnClickListener()
        {
          @Override
          public void onClick(View v)
            { startActivity(new Intent(HoleToPins.this, ApproachFromBelow.class)); }
        }); // set listener
    } // SetUpApproachFromBelow

  private void SetUpLoosestEditBox()
    {
      edLoosest = (EditText) findViewById(R.id.edLoosest);
      edLoosest.addTextChangedListener(new TextWatcher()
        {
          @Override
          public void beforeTextChanged(CharSequence s, int start, int count, int after)
            { } // before text changed; intentionally null

          @Override
          public void onTextChanged(CharSequence s, int start, int before, int count)
            { dmPinGauges.TheConfiguration.Loosest = Float.valueOf(s.toString().trim()); }

          @Override
          public void afterTextChanged(Editable s)
            { } // after text changed; intentionally null
        }); // addTextChangedListener
    } // SetUpLoosestEditBox

  private void SetUpTightestEditBox()
    {
      edTightest = (EditText) findViewById(R.id.edTightest);
      edTightest.addTextChangedListener(new TextWatcher()
        {
          @Override
          public void beforeTextChanged(CharSequence s, int start, int count, int after)
            {  } // before text changed; intentionally null

          @Override
          public void onTextChanged(CharSequence s, int start, int before, int count)
            { dmPinGauges.TheConfiguration.Tightest = Float.valueOf(s.toString().trim()); } // on change

          @Override
          public void afterTextChanged(Editable s)
            { } // after text changed; intentionally null
        }); // addTextChangedListener
    } // SetUpTightestEditBox

  private void SetUpTargetEditBox()
    {
      edTargetHole = (EditText) findViewById(R.id.edTargetHole);
      edTargetHole.addTextChangedListener(new TextWatcher()
        {
          @Override
          public void beforeTextChanged(CharSequence s, int start, int count, int after)
            { } // before text changed; intentionally null

          @Override
          public void onTextChanged(CharSequence s, int start, int before, int count)
            {
              String S = edTargetHole.getText().toString().trim();
              btnFindPins.setEnabled(false);
              if ((S.length() > 0) && !S.equals(PERIOD) && !S.equals(MINUS))
                { btnFindPins.setEnabled(Float.valueOf(S) > 0); }

              lbBestFit.setText(nullstring);
              btnApproachFromBelow.setEnabled(false);
            } // on changed

          @Override
          public void afterTextChanged(Editable s)
            {  } // after text changed; intentionally null
        }); // addTextChangeListener
    } // SetUpTargetEditBox

  private void DismissKeyboard()
    { InputMethodManager TheIMM = (InputMethodManager) getSystemService(INPUT_METHOD_SERVICE);
      if (TheIMM.isAcceptingText()) // dismiss keyboard
        { TheIMM.hideSoftInputFromWindow(getCurrentFocus().getWindowToken(), 0); }
    } // DismissKeyboard

  private void SetUpActionButton()
    {
      btnFindPins = (Button) this.findViewById(R.id.btnFindPins);
      btnFindPins.setEnabled(false);
      btnFindPins.setOnClickListener(new View.OnClickListener()
        {
          @Override
          public void onClick(View v)
            { // Engine does the work
              DismissKeyboard();
              double Target = Float.valueOf(edTargetHole.getText().toString().trim());
              if (Engine.FoundBestFit(Target))
                { lbBestFit.setText(String.format(getString(R.string.BestFit),
                                                  Engine.FormatTupleHole(Engine.FoundNomBigD,
                                                                         Engine.FoundNomMiddleD,
                                                                         Engine.FoundNomSmallD,
                                                                         Engine.ClosestFit
                                                                        )
                                                 ));
                  btnApproachFromBelow.setText(String.format(getString(R.string.ApproachXXFromBelow),
                                                             Target));
                  btnApproachFromBelow.setEnabled(true);
                }
              else
                { lbBestFit.setText(getString(R.string.NoSolutionFound)); }
            } // the listener
        }); // set listener
    } // SetUpActionButton

  private void SetUpBestFit()
    {
      lbBestFit = (TextView) findViewById(R.id.lbBestFit);
    } // SetUpBestFit

  private void LoadUI()
    {
      edLoosest.setText(String.format(Locale.ENGLISH,  "           %.5f", dmPinGauges.TheConfiguration.Loosest));
      edTightest.setText(String.format(Locale.ENGLISH, "           %.5f", dmPinGauges.TheConfiguration.Tightest));
      lbBestFit.setText(nullstring);
    } // LoadUI

} // HoleToPins
