package rrf.pingauges2;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.CursorWrapper;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

import java.util.ArrayList;

/**
 * Created by Robert R. Fenichel on 12/9/2015.
 * data module for the pin-gauges app.
 */

public class dmPinGauges
  {
    private static dmPinGauges PinGaugesDM;

    private static SQLiteDatabase PinGaugesDB;
    public static rrf.pingauges2.tConfiguration TheConfiguration;

    private static final String PERCENT = "%";

    public static dmPinGauges get(Context ThisContext)
      {
        if (PinGaugesDM == null)
          { PinGaugesDM = new dmPinGauges(ThisContext); }
        return PinGaugesDM;
      } // only access to constructor, so it is called only once

    public void LoadDefaults()
      {
        TheConfiguration.IsDefault       = true;
        TheConfiguration.Loosest         = -0.00025;
        TheConfiguration.MaxNomPinOD     = 0.5;
        TheConfiguration.MinNomPinOD     = 0.061;
        TheConfiguration.PinFormatFactor = 1000.0;
        TheConfiguration.PinOffset       = -0.0002;
        TheConfiguration.PinStep         = 0.001;
        TheConfiguration.Tightest        = -0.00015;
        TheConfiguration.ConfigName      = "default";
        TheConfiguration.PinFormat       = "03d";

        tConfigCursorWrapper crSeekDefault = QueryConfigs("IsDefault = ?", new String[] {"1"});
        try
          { if (crSeekDefault.getCount() != 0)
              {  crSeekDefault.moveToFirst();
                 crSeekDefault.FillConfiguration(TheConfiguration);
              }
          }
        finally
          { crSeekDefault.close(); }
      } // LoadDefaults

    public void FillConfigNameList(ArrayList<String> TheList)
      { tConfigCursorWrapper crAll = QueryConfigs(null, null);

        try
          { if (crAll.getCount() != 0)
              {
                crAll.moveToFirst();
                while (!crAll.isAfterLast())
                  { // one record
                    TheList.add(crAll.getString(crAll.getColumnIndex("ConfigName")));
                    crAll.moveToNext();
                  } // one record
              }
          }
        finally
          { crAll.close(); }
      } // FillConfigNameList

    public static String FormatPin(double PinOD)
      { if (TheConfiguration.PinFormatFactor != 1.0)
          { int intPinOD = (int) Math.round(TheConfiguration.PinFormatFactor * PinOD);
            return String.format(PERCENT + TheConfiguration.PinFormat, intPinOD);
          }
        else
          { return String.format(PERCENT + TheConfiguration.PinFormat, PinOD); }
      } // FormatPin

    public void LoadFromDB(String ConfigName)
      { tConfigCursorWrapper TheCursor = QueryConfigs("ConfigName = ?", new String[] {ConfigName});

        try
          { if (TheCursor.getCount() != 0)
              {
                TheCursor.moveToFirst();
                TheCursor.FillConfiguration(TheConfiguration);
              }
          }
        finally
          { TheCursor.close(); }
      } // LoadFromDB

    public int NSavedConfigs()
      { return QueryConfigs(null, null).getCount(); }

    public void SaveToDB(String ConfigName)
      { if (TheConfiguration.IsDefault)
        { // clear any other claim of default
          ContentValues NotDefault = new ContentValues();
          NotDefault.put("IsDefault", 0);
          PinGaugesDB.update("Configurations", NotDefault, null, null);
        } // clear any other claim of default

        tConfigCursorWrapper TheCursor = QueryConfigs("ConfigName = ?", new String[] {ConfigName});

        try
          { if (TheCursor.getCount() == 0)
              { PinGaugesDB.insert("Configurations", null, TheConfiguration.NameValuePairList()); }
            else
              { PinGaugesDB.update("Configurations",
                                   TheConfiguration.NameValuePairList(),
                                   "ConfigName = ?",
                                   new String[] {ConfigName}
                                  );
              }
          }
        finally
          { TheCursor.close(); }
      } // SaveToDB

    private dmPinGauges(Context ThisContext)
      {
        Context LocalContext = ThisContext.getApplicationContext();
        PinGaugesDB = new tPGDBHelper(LocalContext).getWritableDatabase();
        TheConfiguration = new tConfiguration();
        LoadDefaults();
      } // constructor, used only by get

    private tConfigCursorWrapper QueryConfigs(String WhereClause, String[] WhereArgs)
      {
        Cursor RawCursor = PinGaugesDB.query("Configurations",  // table name
                                             null,              // null = all columns selected
                                             WhereClause,
                                             WhereArgs,
                                             null,              // group by
                                             null,              // having
                                             null               //order by
                                            );
        return new tConfigCursorWrapper(RawCursor);
      } // QueryConfigs

    private class tConfigCursorWrapper extends CursorWrapper
      {
        public tConfigCursorWrapper(Cursor ThisCursor)
          { super(ThisCursor); }

        public void FillConfiguration(tConfiguration TheConfiguration)
          {
            TheConfiguration.ConfigName      =  getString(getColumnIndex("ConfigName"    )    );
            TheConfiguration.IsDefault       = (getInt   (getColumnIndex("IsDefault")    ) > 0);
            TheConfiguration.Loosest         =  getDouble(getColumnIndex("Loosest"       )    );
            TheConfiguration.MaxNomPinOD     =  getDouble(getColumnIndex("MaxNomPinOD"   )    );
            TheConfiguration.Tightest        =  getDouble(getColumnIndex("Tightest"      )    );
            TheConfiguration.MinNomPinOD     =  getDouble(getColumnIndex("MinNomPinOD"   )    );
            TheConfiguration.PinFormat       =  getString(getColumnIndex("PinFormat"     )    );
            TheConfiguration.PinFormatFactor = getDouble(getColumnIndex("PinFormatFactor")    );
            TheConfiguration.PinOffset       =  getDouble(getColumnIndex("PinOffset"     )    );
            TheConfiguration.PinStep         =  getDouble(getColumnIndex("PinStep"       )    );
          } // FillConfiguration
      } // tConfigCursorWrapper

    private class tPGDBHelper extends SQLiteOpenHelper
      {
        private static final int DBVersion   = 1;
        private static final String DBName = "PinGauges.db";

        public tPGDBHelper(Context ThisContext)
          { super(ThisContext, DBName, null, DBVersion); } // constructor

        @Override
        public void onCreate(SQLiteDatabase DB)
          {
            DB.execSQL("create table Configurations (           " +
                            "  _id integer primary key autoincrement, " +
                            "  ConfigName       text not null,        " +
                            "  IsDefault        integer,              " +
                            "  Loosest          real not null,        " +
                            "  MaxNomPinOD      real not null,        " +
                            "  MinNomPinOD      real not null,        " +
                            "  PinFormat        text not null,        " +
                            "  PinFormatFactor  real,                 " +
                            "  PinOffset        real not null,        " +
                            "  PinStep          real not null,        " +
                            "  Tightest         real not null         " +
                            "                        )"
                     );
          } // onCreate

        @Override
        public void onUpgrade(SQLiteDatabase DB, int OldVersion, int NewVersion)
          { // just delete & re-create from scratch
            DB.execSQL("drop table if exists " + DBName);
            onCreate(DB);
          } // onUpgrade
      } // tPGDBHelper

  } // dmPinGauges



