unit PlugGaugeMain;
  (* Various calculations for using plug gauges *)

  interface

    uses
      AdvSpin       (* tAdvSpinEdit *),
      Buttons       (* tBitBtn *),
      Classes,
      ComCtrls      (* tPageControl, tTabSheet *),
      Controls      (* crHourglass, tCursor *),
      ExtCtrls      (* tLabeledEdit, tRadioGroup *),
      Forms         (* Application, tCloseAction, tForm, Screen *),
      Mask          (* for AdvSpin *),
      MyUtils       (* tRRFCloseBtn *),
      StdCtrls      (* tButton, tCheckBox, tGroupBox, tListBox *),
      ppBands       (* tppDetailBand *),
      ppCache,
      ppClass       (* tppGroup *),
      ppComm,
      ppCtrls       (* tppDBText *),
      ppDB          (* tppField *),
      ppDBJIT       (* tppJITPipeline *),
      ppDesignLayer (* tppDesignLayer, tppDesignLayers *),
      ppParameter   (* tppParameterList *),
      ppPrnabl,
      ppProd,
      ppRelatv,
      ppReport      (* tppReport *),
      Windows       (**);

    const
      NPinSets = 3;

    type
      tCircumscriber = (ctInfinite, ctLoose, ctOffside, ctOK);

      tPinSet = record
        FirstPin: integer;
        LastPin: integer;
        TheMinimum: integer;
        TheMaximum: integer;
        TheUnit: real;
        TheOffset: real;
        end (* tPinSet *);

    type
      tPinSetArray = array[1 .. NPinSets] of tPinSet;

    type
      tfmPlugGauges = class(tForm)
          pgctlPlugGauges: tPageControl;
            tabSearchFromSets: tTabSheet;
              btnPrint: tBitBtn;
              btnClearSearchResults: tButton;
              btnCopyToClipboard:    tButton;
              grpPinSet1: tGroupBox;
                spinSet1Minimum: tAdvSpinEdit;
                spinSet1Maximum: tAdvSpinEdit;
                ckbxSet1InUse: tCheckBox;
                lbedSet1Offset: tLabeledEdit;
                lbedSet1Unit:   tLabeledEdit;
              grpPinSet2: tGroupBox;
                spinSet2Minimum: tAdvSpinEdit;
                spinSet2Maximum: tAdvSpinEdit;
                ckbxSet2InUse: tCheckBox;
                lbedSet2Offset: tLabeledEdit;
                lbedSet2Unit:   tLabeledEdit;
              grpPinSet3: tGroupBox;
                spinSet3Minimum: tAdvSpinEdit;
                spinSet3Maximum: tAdvSpinEdit;
                ckbxSet3InUse: tCheckBox;
                lbedSet3Offset: tLabeledEdit;
                lbedSet3Unit:   tLabeledEdit;
              grpTargetOne: tGroupBox;
                btnReachFromBelow: tButton;
                btnSearch:         tButton;
                lbedTargetDiameter: tLabeledEdit;
              grpTargetRange: tGroupBox;
                btnStop: tBitBtn;
                btnGoForRange: tButton;
                lbedEndRange:   tLabeledEdit;
                lbedStartRange: tLabeledEdit;
                lbedStepSize:   tLabeledEdit;
              lbedTargetHighTolerance: tLabeledEdit;
              lbedTargetLowTolerance:  tLabeledEdit;
              lbxSearchResults: tListBox;
            tabThreePinsGiven: tTabSheet;
              btnGoGiven3Pins: tButton;
              lbedGivenPin1: tLabeledEdit;
              lbedGivenPin2: tLabeledEdit;
              lbedGivenPin3: tLabeledEdit;
              lbxGivenThreeResults: tListBox;
          btnClose: tRRFCloseBtn;
          rbpipePlugGauges: tppJITPipeline;
            TheString: tppField;
            reportPlugGauges: tppReport;
              rbparamsPlugGauges: tppParameterList;
              rbdeslayersPlugGauges: tppDesignLayers;
                rbdeslayerPlugGauges: tppDesignLayer;
              rbbdDetail: tppDetailBand;
                rbdbtxTheString: tppDBText;
          rgShowPinDiameter: tRadioGroup;
              (* eject *)
          procedure btnClearSearchResultsClick(Sender: tObject);
          procedure btnCopyToClipboardClick   (Sender: tObject);
          procedure btnGoForRangeClick        (Sender: tObject);
          procedure btnGoGiven3PinsClick      (Sender: tObject);
          procedure btnPrintClick             (Sender: tObject);
          procedure btnReachFromBelowClick    (Sender: tObject);
          procedure btnSearchClick            (Sender: tObject);
          procedure btnStopClick              (Sender: tObject);
          procedure FormShow                  (Sender: tObject);
          procedure lbedGivenPinChange        (Sender: tObject);
          procedure SearchUIChanged           (Sender: tObject);

          procedure FormClose(Sender: tObject; var Action: tCloseAction);
          function rbpipePlugGaugesGetFieldValue(aFieldName: string): variant;
        private
          BreakRequested    (* huge search started inadvertently    *): boolean;
          TotalPins         (* set 3 is LastSet1Pin+1, ..., this    *): integer;
          FoundBigDIndex    (* from btnSearch                       *): integer;
          ReportingInterval (* if NZ, force reporting during search *): longint;
          PinSet            (* from the UI                          *): tPinSetArray;
          FoundBigD         (* from btnSearch                       *): real;
          FoundMiddleD      (* from btnSearch                       *): real;
          FoundSmallD       (* from btnSearch                       *): real;
          HighTolerance     (* maximum acceptable overage           *): real;
          LowTolerance      (* minimum acceptable underage          *): real;
          Target            (* nominal target diameter for search   *): real;
          function  CircumscribeThree(const D1, D2, D3: real;
                                      out ResultKind: tCircumscriber): real;
          function  DescribeResult(const D1, D2, D3, D: real;
                                   const ResultKind: tCircumscriber): string;
          function  FoundBestFit(const Goal: real; out
                                 DResult, Pin1, Pin2, Pin3: real): boolean;
          function  GuardedStrToFloat(const S: string; out Converted: boolean): real;
          function  PinDiameter(const Index: integer): real;
          procedure Validate;
        end (* tfmPlugGauges *);

    var
      fmPlugGauges: tfmPlugGauges;

  implementation
    uses
      Clipbrd  (* Clipboard *),
      MyConsts (* nullstring *),
      Registry (* tRegistry *),
      SysUtils (* Abort, EConvertError, Format *);
      {$R *.dfm}

    const
      CreateIfMissing = true;
      PlugGaugeSet1Key    = 'Software\RRF\PlugGauges\Set1';
      PlugGaugeSet2Key    = 'Software\RRF\PlugGauges\Set2';
      PlugGaugeSet3Key    = 'Software\RRF\PlugGauges\Set3';
      InUseKey        = 'InUse';
      MinKey          = 'minimum';
      MaxKey          = 'maximum';
      UnitKey         = 'unit';
      OffsetKey       = 'offset';

    procedure tfmPlugGauges.btnClearSearchResultsClick(Sender: tObject);
      (* housekeeping *)

      begin (* tfmPlugGauges.btnClearSearchResultsClick *)
        lbxSearchResults.Clear
      end   (* tfmPlugGauges.btnClearSearchResultsClick *);
      (* eject *)                                                {$WARNINGS OFF}
    procedure tfmPlugGauges.btnCopyToClipboardClick(Sender: tObject);
      (* facilitate off-app use of table *)

      begin (* tfmPlugGauges.btnCopyToClipboardClick *)
        Clipboard.SetTextBuf(pchar(lbxSearchResults.Items.Text))
      end   (* tfmPlugGauges.btnCopyToClipboardClick *);
                                                                 {$WARNINGS ON}
    procedure tfmPlugGauges.btnGoGiven3PinsClick(Sender: tObject);
      (* compute circumscribed circle for pins described in the 3
       labeled edit boxes.  Button disabled when good numbers not
       available *)

      var
        ResultKind: tCircumscriber;
        Circumscriber, D1, D2, D3: real;

      begin (* tfmPlugGauges.btnGoGiven3PinsClick *)
        D1 := StrToFloat(lbedGivenPin1.Text);
        D2 := StrToFloat(lbedGivenPin2.Text);
        D3 := StrToFloat(lbedGivenPin3.Text);

        Circumscriber := CircumscribeThree(D1, D2, D3, ResultKind);

        with lbxGivenThreeResults do
          ItemIndex := Items.Add(DescribeResult(D1, D2, D3,
                                                Circumscriber,
                                                ResultKind))
      end   (* tfmPlugGauges.btnGoGiven3PinsClick *);

    procedure tfmPlugGauges.btnGoForRangeClick(Sender: tObject);
      (* Run over the range defined in the grpTargetRange controls.  For
      each target size, generate one line for the best-fitting solution *)

      var
        EndOK, StartOK, StepOK: boolean;
        OldCursor: tCursor;
        I: integer;
        BigD, DResult, EndRange, MiddleD, SmallD, StartRange, StepSize, ThisTarget: real;
        Line: string;

      procedure DescribePinSet(const I: integer);
        (* Annotate the listbox with a description of the pins in use *)

        var
          Offset: string;

        begin (* DescribePinSet *)
          with PinSet[I], lbxSearchResults do
            begin (* PinSet[I], lbxSearchResults *)
              if TheOffset > 0 then
                Offset := Format(' (plus %.4f)', [TheOffset])
              else if TheOffset < 0 then
                Offset := Format(' (minus %.4f)', [-TheOffset])
              else
                Offset := nullstring;

              ItemIndex := Items.Add(Format('Pins %.4f to %.4f by %.4f%s',
                                            [TheUnit * TheMinimum,
                                             TheUnit * TheMaximum,
                                             TheUnit,
                                             Offset]))
            end   (* PinSet[I], lbxSearchResults *)
        end   (* DescribePinSet *);

      begin (* tfmPlugGauges.btnGoForRangeClick *)
        BreakRequested := false;
        OldCursor := Screen.Cursor;
        Screen.Cursor := crHourglass;
        StartRange := GuardedStrToFloat(lbedStartRange.Text, StartOK);
        EndRange   := GuardedStrToFloat(lbedEndRange.Text,   EndOK);
        StepSize   := GuardedStrToFloat(lbedStepSize.Text,   StepOK);
        I := 1;

        if StartOK then
          if EndOK then
            if StepOK then
              with lbxSearchResults do
                try     (* restore cursor *)
                  if ckbxSet1InUse.Checked then
                    DescribePinSet(1);
                  if ckbxSet2InUse.Checked then
                    DescribePinSet(2);
                  if ckbxSet3InUse.Checked then
                    DescribePinSet(3);
                  ItemIndex := Items.Add(Format('tolerance: %.6f to %.6f',
                                                [LowTolerance, HighTolerance]));
                  ItemIndex := Items.Add(nullstring);
                  ThisTarget := StartRange;

                  while (ThisTarget >= StartRange) and (ThisTarget <= EndRange) do
                    begin (* one target *)
                      if FoundBestFit(ThisTarget, DResult, BigD, MiddleD, SmallD) then
                        Line := Format('%.4f: best fit found was %.4f, %.4f, %.4f [%.6f]',
                                       [ThisTarget, BigD, MiddleD, SmallD, DResult])
                      else
                        Line := Format('%.4f: no fit found with these pins & tolerances', [ThisTarget]);

                      case I mod 5 of
                        0, 2,
                        3, 4: begin (* allow break *)
                                sleep(1);
                                Application.ProcessMessages;
                                if BreakRequested then
                                  break
                              end   (* allow break *);
                           1: ItemIndex := Items.Add(nullstring)
                        end (* case *);
                      ItemIndex := Items.Add(Line);

                      ThisTarget := ThisTarget + StepSize;
                      inc(I)
                    end   (* one target *)
                finally (* restore cursor *)
                  Screen.Cursor := OldCursor
                end     (* restore cursor *)
      end   (* tfmPlugGauges.btnGoForRangeClick *);

    procedure tfmPlugGauges.btnPrintClick(Sender: tObject);
      (* print the list box *)
    
      begin (* tfmPlugGauges.btnPrintClick *)
        rbpipePlugGauges.InitialIndex := 1;
        rbpipePlugGauges.RecordCount := lbxSearchResults.Count;
        reportPlugGauges.Print
      end   (* tfmPlugGauges.btnPrintClick *);

    procedure tfmPlugGauges.btnReachFromBelowClick(Sender: tObject);
      (* Tabulate all of the possible combos using FoundSmallD and FoundMiddleD *)

      var
        LinesOut, Runner: integer;

      procedure DescribeThisOne(const BigIndex: integer);
        (* if this one qualifies, list it *)

        var
          ResultKind: tCircumscriber;
          TentativeD, TentativeBig: real;

        begin (* DescribeThisOne *)
          if BigIndex > 0 then
            begin (* nondegenerate *)
              TentativeBig := PinDiameter(BigIndex);
              TentativeD := CircumscribeThree(TentativeBig, FoundMiddleD, FoundSmallD, ResultKind);
              if ResultKind = ctOK then
                begin (* spaced lines *)
                  if (LinesOut mod 4) = 0 then
                    with lbxSearchResults do
                      ItemIndex := Items.Add(nullstring);
                  with lbxSearchResults do
                    ItemIndex := Items.Add(DescribeResult(TentativeBig,
                                                          FoundMiddleD,
                                                          FoundSmallD,
                                                          TentativeD,
                                                          ctOK));
                  inc(LinesOut)
                end   (* spaced lines *)
            end   (* nondegenerate *)
        end   (* DescribeThisOne *);

      begin (* tfmPlugGauges.btnReachFromBelowClick *)
        LinesOut := 0;
        Runner := FoundBigDIndex mod 100;
        repeat (* step by 0.1" *)
          DescribeThisOne(Runner);
          inc(Runner, 100)
        until  (* step by 0.1" *) Runner >= FoundBigDIndex;

        dec(Runner, 90);
        repeat (* step by 0.01" *)
          DescribeThisOne(Runner);
          inc(Runner, 10)
        until  (* step by 0.01" *) Runner >= FoundBigDIndex;

        dec(Runner, 9);
        repeat (* step by 0.001" *)
          DescribeThisOne(Runner);
          inc(Runner, 1)
        until  (* step by 0.001" *) Runner > FoundBigDIndex
      end   (* tfmPlugGauges.btnReachFromBelowClick *);

    procedure tfmPlugGauges.btnSearchClick(Sender: tObject);
      (* Find the best approximation to the desired target diameter, reporting
      all results within the given tolerance band and reporting every so often
      in any event. *)

      var
        FoundAFit: boolean;
        OldCursor: tCursor;
        D: real;
        Line: string;

      begin (* tfmPlugGauges.btnSearchClick *)
        OldCursor := Screen.Cursor;
        Screen.Cursor := crHourglass;
        try     (* restore cursor *)
          FoundAFit := FoundBestFit(Target, D, FoundBigD, FoundMiddleD, FoundSmallD);
          btnReachFromBelow.Enabled := FoundAFit;
          
          if FoundAFit then
            Line := DescribeResult(FoundBigD, FoundMiddleD, FoundSmallD, D, ctOK)
          else
            Line := 'no solution found';

          with lbxSearchResults do
            ItemIndex := Items.Add(Line)
        finally (* restore cursor *)
          Screen.Cursor := OldCursor
        end     (* restore cursor *)
      end   (* tfmPlugGauges.btnSearchClick *);
      (* eject *)
    procedure tfmPlugGauges.btnStopClick(Sender: tObject);
      (* get the search to abort *)

      begin (* tfmPlugGauges.btnStopClick *)
        BreakRequested := true
      end   (* tfmPlugGauges.btnStopClick *);

    function tfmPlugGauges.CircumscribeThree(const D1, D2, D3: real;
                                             out ResultKind: tCircumscriber): real;
      (* return the diameter of the circle that circumscribes the mutually
      tangent circles with the given diameters.  If there is no circumscribing
      tangent circle, or only a loose one, then those results are reported
      as ResultKind. *)

      var
        K1, K2, K3, K4 (* curvatures *): real;

      begin (* tfmPlugGauges.CircumscribeThree *)
        Result := 0 (* suppress compiler warning *);
        ResultKind := ctOK;

        K1 := 2.0 / D1;
        K2 := 2.0 / D2;
        K3 := 2.0 / D3;
        K4 := K1 + K2 + K3 - 2.0 * sqrt(K1 * K2 + K2 * K3 + K3 * K1);

        if K4 = 0 then
          ResultKind := ctInfinite
        else
          Result := -2.0 / K4;

        if K4 > 0 then
          ResultKind := ctOffside
        else if Result > D1 + D2 + D3 then
          ResultKind := ctLoose
      end   (* tfmPlugGauges.CircumscribeThree *);

    function tfmPlugGauges.DescribeResult(const D1, D2, D3, D: real;
                                          const ResultKind: tCircumscriber): string;
      (* human-readable version of a result *)

      begin (* tfmPlugGauges.DescribeResult *)
        case rgShowPinDiameter.ItemIndex of
          0: Result := Format('(%d, %d, %d) --> %.4f',
                              [trunc(1000*D1+0.5),
                               trunc(1000*D2+0.5),
                               trunc(1000*D3+0.5),
                               D]);
          1: Result := Format('(%.3f, %.3f, %.3f) --> %.4f', [D1, D2, D3, D]);
          2: Result := Format('(%.4f, %.4f, %.4f) --> %.4f', [D1, D2, D3, D])
          end (* case *);

        case ResultKind of
          ctInfinite: Result := Format('(%.4f, %.4f, %.4f) tops colinear', [D1, D2, D3]);
             ctLoose: Result := Result + ' loose';
           ctOffside: Result := Result + ' offside'
          end (* case *)
      end   (* tfmPlugGauges.DescribeResult *);
      
    procedure tfmPlugGauges.FormClose(Sender: tObject;
                                      var Action: tCloseAction);
      (* save pin sets *)

      procedure SaveUI(TheKey: string;
                       InUse: tCheckBox;
                       SpinMin, SpinMax: tAdvSpinEdit;
                       lbedUnit, lbedOffset: tLabeledEdit);
        (* save one pin set *)

        begin (* SaveUI *)
          with tRegistry.Create do
            try     (* using the registry *)
              if OpenKey(TheKey, CreateIfMissing) then
                begin (* save this set *)
                  WriteBool(InUseKey, InUse.Checked);
                  WriteInteger(MinKey, SpinMin.Value);
                  WriteInteger(MaxKey, SpinMax.Value);
                  WriteString(UnitKey, lbedUnit.Text);
                  WriteString(OffsetKey, lbedOffset.Text)
                end   (* save this set *)
            finally (* using the registry *)
              Free
            end     (* using the registry *)
        end   (* SaveUI *);

      begin (* tfmPlugGauges.FormClose *)
        SaveUI(PlugGaugeSet1Key, ckbxSet1InUse, spinSet1Minimum, spinSet1Maximum,
               lbedSet1Unit, lbedSet1Offset);
        SaveUI(PlugGaugeSet2Key, ckbxSet2InUse, spinSet2Minimum, spinSet2Maximum,
               lbedSet2Unit, lbedSet2Offset);
        SaveUI(PlugGaugeSet3Key, ckbxSet3InUse, spinSet3Minimum, spinSet3Maximum,
               lbedSet3Unit, lbedSet3Offset)
      end   (* tfmPlugGauges.FormClose *);
      (* eject *)
    procedure tfmPlugGauges.FormShow(Sender: tObject);
      (* Initialization *)

      procedure LoadUI(TheKey: string;
                       InUse: tCheckBox;
                       SpinMin, SpinMax: tAdvSpinEdit;
                       lbedUnit, lbedOffset: tLabeledEdit);
        (* load up one pin set *)

        begin (* LoadUI *)
          with tRegistry.Create do
            try     (* using the registry *)
              if OpenKey(TheKey, not CreateIfMissing) then
                begin (* get data for this set *)
                  if ValueExists(InUseKey) then
                    InUse.Checked := ReadBool(InUseKey);
                  if ValueExists(MinKey) then
                    SpinMin.Value := ReadInteger(MinKey);
                  if ValueExists(MaxKey) then
                    SpinMax.Value := ReadInteger(MaxKey);
                  if ValueExists(UnitKey) then
                    lbedUnit.Text := ReadString(UnitKey);
                  if ValueExists(OffsetKey) then
                    lbedOffset.Text := ReadString(OffsetKey)
                end   (* get data for this set *)
            finally (* using the registry *)
               Free
            end     (* using the registry *)
        end   (* LoadUI *);

      begin (* tfmPlugGauges.FormShow *)
        ReportingInterval := 0 (* NZ only for debugging *);
        LoadUI(PlugGaugeSet1Key, ckbxSet1InUse, spinSet1Minimum, spinSet1Maximum,
               lbedSet1Unit, lbedSet1Offset);
        LoadUI(PlugGaugeSet2Key, ckbxSet2InUse, spinSet2Minimum, spinSet2Maximum,
               lbedSet2Unit, lbedSet2Offset);
        LoadUI(PlugGaugeSet3Key, ckbxSet3InUse, spinSet3Minimum, spinSet3Maximum,
               lbedSet3Unit, lbedSet3Offset);
        btnReachFromBelow.Enabled := false;

        Validate
      end   (* tfmPlugGauges.FormShow *);
      (* eject *)
    function tfmPlugGauges.FoundBestFit(const Goal: real;
                                        out DResult, Pin1, Pin2, Pin3: real): boolean;
      (* Hold out for the best fit. A trick here is that all the Cartesian
      calculation gives is a tangent circle, but it may not be a tight one.
      For example, consider 3 equal, tangent circles, and shrink one of them
      slightly.  The radius of a circumscribing circle will expand, but of course
      the new, smaller circle will fit (loosely) in the old circumscriber.  The
      new circle is acceptable only if it is reached through reducing a
      base circle and is SMALLER than the circumscriber for the set that
      included the larger base. *)

      const
        VeryBig = 9999;

      var
        ResultKind: tCircumscriber;
        BigIndex, MiddleIndex, SmallIndex: integer;
        BigD, D, Delta, HighAcceptable, LowAcceptable, MiddleD, NewDelta,
          OldD, RangeMiddle, SmallD: real;

      begin (* tfmPlugGauges.FoundBestFit *)
        Delta := VeryBig;
        HighAcceptable := Goal + HighTolerance;
        LowAcceptable := Goal + LowTolerance;
        RangeMiddle := (LowAcceptable + HighAcceptable) / 2;
        Result := false;

        for BigIndex := TotalPins downto 1 do
          begin (* big pin chosen *)
            BigD := PinDiameter(BigIndex);
            if BigD < HighAcceptable then
              for MiddleIndex := BigIndex-1 downto 1 do
                begin (* middle pin chosen *)
                  MiddleD := PinDiameter(MiddleIndex);
                  OldD := VeryBig;
                  if BigD + MiddleD < HighAcceptable then
                    for SmallIndex := MiddleIndex-1 downto 1 do
                      begin (* small pin chosen *)
                        SmallD := PinDiameter(SmallIndex);
                        if BigD + MiddleD + SmallD < LowAcceptable then
                          break;
                        D := CircumscribeThree(BigD, MiddleD, SmallD, ResultKind);
                        if D > OldD then (* must be loose if D bigger when SmallD bigger *)
                          break;
                        OldD := D;
                        if ResultKind = ctOK then
                          begin (* not silly *)
                            NewDelta := abs(RangeMiddle - D);
                            if NewDelta < Delta then
                              begin (* closing in *)
                                Delta := NewDelta;
                                if D <= HighAcceptable then
                                  if D >= LowAcceptable then
                                    begin (* got one *)
                                      DResult := D;
                                      Pin1 := BigD;
                                      Pin2 := MiddleD;
                                      Pin3 := SmallD;
                                      FoundBigDIndex := BigIndex;
                                      Result := true
                                    end   (* got one *)
                              end   (* closing in *)
                          end   (* not silly *)
                      end   (* small pin chosen *)
                end   (* middle pin chosen *)
          end   (* big pin chosen *)
      end   (* tfmPlugGauges.FoundBestFit *);

    function tfmPlugGauges.GuardedStrToFloat(const S: string;
                                             out Converted: boolean): real;
      (* fragile conversion *)

      begin (* tfmPlugGauges.GuardedStrToFloat *)
        Result := 0 (* avoid compiler message *);
        Converted := true;
        try    (* float conversion is fragile *)
          Result := StrToFloat(S)
        except (* float conversion is fragile *)
          on E: EConvertError do
            Converted := false
        end    (* float conversion is fragile *)
      end   (* tfmPlugGauges.GuardedStrToFloat *);

    procedure tfmPlugGauges.lbedGivenPinChange(Sender: tObject);
      (* if not convertible to a positive real number, disable computation *)

      var
        Converted: boolean;
        Value: real;

      begin (* tfmPlugGauges.lbedGivenPinChange *)
        Value := GuardedStrToFloat(tLabeledEdit(Sender).Text, Converted);
        btnGoGiven3Pins.Enabled := Converted and (Value > 0)
      end   (* tfmPlugGauges.lbedGivenPinChange *);

    function tfmPlugGauges.PinDiameter(const Index: integer): real;
      (* get the diameter by first figuring out which set the pin is from *)

      var
        I: integer;

      begin (* tfmPlugGauges.PinDiameter *)
        Result := 0;
        for I := 1 to NPinSets do
          with PinSet[I] do
            if Index <= LastPin then
              begin (* use this one *)
                Result := (TheMinimum + Index - FirstPin) * TheUnit + TheOffset;
                break
              end   (* use this one *)
     end   (* tfmPlugGauges.PinDiameter *);

    function tfmPlugGauges.rbpipePlugGaugesGetFieldValue(aFieldName: string): variant;
      (* uses the listbox as client dataset *)
    
      begin (* tfmPlugGauges.rbpipePlugGaugesGetFieldValue *)
        Result := lbxSearchResults.Items[rbpipePlugGauges.RecordIndex-1]
      end   (* tfmPlugGauges.rbpipePlugGaugesGetFieldValue *);

    procedure tfmPlugGauges.SearchUIChanged(Sender: tObject);
      (* time to see if search would now be meaningful *)

      begin (* tfmPlugGauges.SearchUIChanged *)
        Validate
      end   (* tfmPlugGauges.SearchUIChanged *);
      (* eject *)
    procedure tfmPlugGauges.Validate;
      (* Enable or disable the Search buttons after looking at the UI *)

      var
        PinsAreOK, Converted1, Converted2: boolean;

      procedure CountPinSet(const SetNumber: integer;
                           spinMinimum, spinMaximum: tAdvSpinEdit;
                           lbedUnit, lbedOffset: tLabeledEdit);
        (* Count pins in set defined by those controls *)

        var
          Converted: boolean;

        begin (* CountPinSet *)
          with PinSet[SetNumber] do
            begin (* load pin set *)
              FirstPin := TotalPins + 1;
              TheMinimum := spinMinimum.Value;
              TheMaximum := spinMaximum.Value;
              LastPin := FirstPin + TheMaximum - TheMinimum;

              if (TheMinimum <= 0) or (TheMinimum > TheMaximum) then
                PinsAreOK := false;

              TheUnit := GuardedStrToFloat(lbedUnit.Text, Converted);
              if not Converted then
                PinsAreOK := false;

              TheOffset := GuardedStrToFloat(lbedOffset.Text, Converted);
              if not Converted then
                PinsAreOK := false;

              inc(TotalPins, LastPin - FirstPin + 1)
            end   (* load pin set *)
        end   (* CountPinSet *);

      begin (* tfmPlugGauges.Validate *)
        btnSearch.Enabled := false;
        TotalPins := 0;
        PinsAreOK := true;

        if ckbxSet1InUse.Checked then
          CountPinSet(1, spinSet1Minimum, spinSet1Maximum,
                         lbedSet1Unit, lbedSet1Offset);
        if ckbxSet2InUse.Checked then
          CountPinSet(2, spinSet2Minimum, spinSet2Maximum,
                         lbedSet2Unit, lbedSet2Offset);
        if ckbxSet3InUse.Checked then
          CountPinSet(3, spinSet3Minimum, spinSet3Maximum,
                         lbedSet3Unit, lbedSet3Offset);

        LowTolerance  := GuardedStrToFloat(lbedTargetLowTolerance.Text,  Converted1);
        HighTolerance := GuardedStrToFloat(lbedTargetHighTolerance.Text, Converted2);
        if Converted1 and Converted2 and PinsAreOK and (TotalPins > 0) then
          begin (* buttons possible *)
            Target := GuardedStrToFloat(lbedTargetDiameter.Text, Converted1);
            if Converted1 then
              btnSearch.Enabled := true
          end   (* buttons possible *)
      end   (* tfmPlugGauges.Validate *);

end.
